#include "device.h"

#define MODE_FULLON                    0x00000001   /* Mask for FULL-ON Mode for DPM0_STAT register */
#define MODE_ACTIVE                    0x00000002   /* Mask for ACTIVE Mode for DPM0_STAT register */
void Active_To_Fullon(void);

__ramfunc void Init_PLL(char MSEL, char DF, char CSEL, char SSEL)
{	
	
	Active_To_Fullon(); // Upon reset, processor will be in Active mode. Change to Full on before configuring the CGU 

	/* check if PLL is enabled , PLL is not locking, clocks are aligned */
	while( !((*pREG_CGU0_STAT & (BITM_CGU_STAT_CLKSALGN | BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_PLLEN)) == (BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_PLLEN) ) );
			
	if(	(((*pREG_CGU0_CTL & BITM_CGU_CTL_DF  ) >> BITP_CGU_CTL_DF  ) == DF  ) && 
		(((*pREG_CGU0_CTL & BITM_CGU_CTL_MSEL) >> BITP_CGU_CTL_MSEL) == MSEL) )
		{
			*pREG_CGU0_DIV = (	((CSEL 	<< BITP_CGU_DIV_CSEL  ) & BITM_CGU_DIV_CSEL  ) |
								((SSEL 	<< BITP_CGU_DIV_SYSSEL) & BITM_CGU_DIV_SYSSEL) |
								((10 	<< BITP_CGU_DIV_OSEL  ) & BITM_CGU_DIV_OSEL) ) | BITM_CGU_DIV_UPDT;
		}
	else
		{
			*pREG_CGU0_DIV = (	((CSEL 	<< BITP_CGU_DIV_CSEL  ) & BITM_CGU_DIV_CSEL  ) |
								((SSEL 	<< BITP_CGU_DIV_SYSSEL) & BITM_CGU_DIV_SYSSEL) |
								((10 	<< BITP_CGU_DIV_OSEL  ) & BITM_CGU_DIV_OSEL  ) ) ;
			*pREG_CGU0_CTL = ( 	((MSEL  << BITP_CGU_CTL_MSEL  ) & BITM_CGU_CTL_MSEL  ) | 
		  						((DF    << BITP_CGU_CTL_DF    ) & BITM_CGU_CTL_DF    ) );
		}
		*pREG_CGU0_CLKOUTSEL = ((3 << BITP_CGU_CLKOUTSEL_CLKOUTSEL) & BITM_CGU_CLKOUTSEL_CLKOUTSEL) ;
		
	/* wait till PLL is locked and clocks are aligned */
	while( !((*pREG_CGU0_STAT & (BITM_CGU_STAT_CLKSALGN | BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_PLLEN)) == (BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_PLLEN) ) );
	asm("nop;");
}



__ramfunc void Active_To_Fullon()
{
//Create variables to hold the new values
	uint32_t oldDPM0_STAT, oldCGU0_STAT ,newDPM0_CTL;

	//Load DMP0_STAT and CGU0_STAT values to variables
	oldDPM0_STAT = *pREG_DPM0_STAT;
	oldCGU0_STAT = *pREG_CGU0_STAT;

	// Check for proper pre-switch values of DPM0_STAT and CGU0_STAT registers
	if ((oldDPM0_STAT & MODE_ACTIVE) && (oldCGU0_STAT & BITM_CGU_STAT_PLLBP))
	{
		newDPM0_CTL = BITM_DPM_CTL_PLLBPCL;
		*pREG_DPM0_CTL = newDPM0_CTL;

		//Load DMP0_STAT and CGU0_STAT values to variables
		oldDPM0_STAT = *pREG_DPM0_STAT;
		oldCGU0_STAT = *pREG_CGU0_STAT;

		while(!((oldDPM0_STAT & MODE_FULLON) && ~(oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT & BITM_CGU_STAT_CLKSALGN)));
	}
}
